{*********************************************************}
{* StarAnim: Simulation of our solar system    22.01.92  *}
{* ----------------------------------------------------- *}
{* orig.: AndreTheiler & CarstenWartmann, Amiga Modula-2 *}
{* adaption to Turbo Pascal 5.5 by Gerd Platl  02.07.98  *}
{*********************************************************}
{$R-,S-,D-,L-,I-,F-,N+,E-}

PROGRAM SolarSystemSimulation;

uses  Crt, Graph, BGI16;

const datFile = 'SolSySim.DAT';   {english version}
     {datFile = 'SolSySim.GER';   {german version}

{*********************************************************}

const maxStars = 20;
      gKonst = 1.487661E-25;      {gravitation constant}

      cb: char = #0;
      starNo: integer = 4;        {objects of DAT file}
      starSN: integer = 4;        {objects to simulate}

type  {float = real;}
      {float = single;}
      float = double;
      {float = extended;}

type  tStarData = record
        name: string[20];
        color, sx, sy: integer;
        x,y,z, masse, vx,vy,vz: float;
        zf, dt: real;
      end;

var   xmid, ymid: integer;
      zoom, deltat: float;
      vAngle, sa,ca: float;
      starData: array [0..maxStars] of tStarData;

{*********************************************************}
function ReadStarData: boolean;

var   si: word;   data: text;   ioError: integer;

begin
  {$I-}
  Assign (data, datFile);
  Reset (data);
  ioError := ioResult;

  if ioError <> 0
  then begin
    ReadStarData := false;
    writeln ('Error ', ioError, ': at open file ',datFile);
    exit;
  end;

  readln (data, starNo);
  si := 0;
  while (si < starNo) and not eof (data)
  do with starData[si]
  do begin
    readln (data, name);
    readln (data, name);
    if pos (' ',name) <> 0 then name[0] := char(pos(' ',name)-1);
    readln (data, color, zf, dt);
    readln (data, x, y, z);
    readln (data, vx, vy, vz);
    readln (data, masse);
    masse := masse * gKonst;
    inc (si);
    TextColor (color);
    writeln (si:4, ': ',name:12,masse:12:4);
  end;
  Close (data);
  ReadStarData := true;
end;
{*********************************************************}
procedure SetViewAngle (grad: float);
const deg   = Pi / 180.0;         {1 / Radiant = 0.017453292...}
begin
  vAngle := grad;
  grad := grad * deg;
  sa := sin(grad);
  ca := cos(grad);
end;
{*********************************************************}
procedure SetObjectNo (no: integer);
begin
  if (no > 0) and (no <= starNo) then starSN := no;
  zoom   := starData[starSN-1].zf;
  deltat := starData[starSN-1].dt;
end;
{*********************************************************}
procedure DrawStar (var starData: tStarData);

begin
  with starData
  do begin
    sx := xmid + TRUNC (x * zoom);
    sy := ymid - TRUNC (y * zoom * sa
                       +z * zoom * ca);
    SetColor (color);
    Line (sx-2, sy+2, sx+2, sy+2);
    Line (sx-2, sy-2, sx+2, sy-2);
    Line (sx+2, sy-2, sx+2, sy+2);
    Line (sx-2, sy-2, sx-2, sy+2);
  end;
end;
{*********************************************************}
procedure StarDance;

var   s1,s2, year, yday: integer;
      hs,ds: string[20];
      dx,dy,dz, dq, fa, ax,ay,az, days: real;

  procedure DrawStarScreen;

  var  si: integer;

  begin
    ClearViewPort;
    str (zoom:9:4, hs);
    SetColor (darkgray);
    OuttextXY (1,12,'+ -  Zoom ' + hs);

    str (deltat:7:3, hs);
    SetColor (lightgray);
    OuttextXY (1,24,'* /  delta_t' + hs);

    str (vAngle:7:0, hs);
    SetColor (darkgray);
    OuttextXY (1,36,'   vAngle' + hs+'');
    OuttextXY (1,48,' '#26'  satellites');

    for si := 0 to starSN - 1
    do with starData[si]
    do begin
      SetColor (color);
      OuttextXY (1, 12 * (5 + si), name);
      DrawStar (starData[si]);
    end;
    cb := #0;
  end;

begin
  SetObjectNo (5);
  days := 0;
  xmid := (GetMaxX + 1) div 2;
  ymid := (GetMaxY + 1) div 2;
  SetFillStyle (solidFill, Black);
  SetWriteMode (XOrPut);
  DrawStarScreen;
  repeat
    s1 := 0;
    while (s1 < starNo)
    do with starData[s1]
    do begin
      ax := 0.0;
      ay := 0.0;
      az := 0.0;
      s2 := 0;

      if s1 < starSN
      then DrawStar (starData[s1]);    {clear}
      while (s2 < starNo)
      do begin
        if (s1 <> s2)
        then begin
          dx := starData[s2].x - x;
          dy := starData[s2].y - y;
          dz := starData[s2].z - z;

          dq := dx*dx + dy*dy + dz*dz; {distance^2}
          fa := starData[s2].masse / (dq*sqrt(dq));

          ax := ax + dx * fa;
          ay := ay + dy * fa;
          az := az + dz * fa;
        end;
        inc (s2);
      end;

      vx := vx + ax * deltat;
      vy := vy + ay * deltat;
      vz := vz + az * deltat;

      x := x + vx * deltat;
      y := y + vy * deltat;
      z := z + vz * deltat;

      PutPixel (sx, sy, darkgray);
      if s1 < starSN
      then DrawStar (starData[s1]);    {draw}
      inc (s1);
    end;

    if keyPressed
    then begin
      cb := readkey;
      if cb = ' ' then DrawStarScreen else
      if cb = '+' then begin   zoom :=   zoom * 1.01;  DrawStarScreen; end else
      if cb = '-' then begin   zoom :=   zoom / 1.01;  DrawStarScreen; end;
      if cb = '*' then begin deltat := deltat * 1.01;  DrawStarScreen; end else
      if cb = '/' then begin deltat := deltat / 1.01;  DrawStarScreen; end;
      if cb = 'h' then begin SetViewAngle (0);         DrawStarScreen; end else
      if cb = 'v' then begin SetViewAngle (90);        DrawStarScreen; end;
      if cb = 'H' then begin SetViewAngle (vAngle+1);  DrawStarScreen; end else
      if cb = 'P' then begin SetViewAngle (vAngle-1);  DrawStarScreen; end;
      if cb = 'M' then begin SetObjectNo (starSN+1);   DrawStarScreen; end else
      if cb = 'K' then begin SetObjectNo (starSN-1);   DrawStarScreen; end;
      {OuttextXY (1,1,cb);}
    end;
    days := days + deltat;
    year := 1992 + trunc (days / 365);
    yday := 1+(trunc (days) mod 365) div 31;
    str (year:4, hs);
    str (yday:4, ds);
    Bar (80,0, 152,8);
    SetColor (lightGray);
    OuttextXY (1,1,'     date '+ds+'/'+hs);
  until cb = #27;
end;
{*********************************************************}
begin
  TextColor (yellow);
  writeln (#13#10'--- SolSySim V1.2 ---'#13#10);
  TextColor (white);
  writeln ('  * /   time delta in days'#13#10
          ,'  + -   zoom'#13#10
          ,'  v h   vert./horiz. view'#13#10
          ,' blanc  redraw'#13#10
          ,'  ESC   quit'#13#10);
  if ReadStarData
  then begin
    cb := readkey;

    InitGraphics (VGA640x480x16);

    SetViewAngle (90);
    StarDance;

    CloseGraphics;
  end;
end.
